import { Message, MessageCode, MessageSeverity } from "./message";
import { Result } from "./result";

// tslint:disable-next-line:class-name
export class cLog {

    /** pointer to parent result */
    public parent: Result;

    private _messages: Message[] = [];

    public constructor(parent?: Result) {
        this.parent = parent;
    }

    /**
     * efface tous les messages
     */
    public clear() {
        this._messages = [];
    }

    /**
     * insère un message en fin de liste
     */
    public add(m: Message) {
        // add pointer to the current log as message's parent, unless the message
        // already has a parent, and unless the current log has no pointer to a result
        if (m.parent === undefined && this.parent !== undefined) {
            m.parent = this;
        }
        this._messages.push(m);
    }

    /**
     * insère un log entier en fin de liste
     */
    public addLog(l: cLog) {
        for (const m of l.messages) {
            this.add(m);
        }
    }

    /**
     * insère un message en début de liste
     */
    public insert(m: Message) {
        // add pointer to the current log as message's parent, unless the message
        // already has a parent, and unless the current log has no pointer to a result
        if (m.parent === undefined && this.parent !== undefined) {
            m.parent = this;
        }
        this._messages.unshift(m);
    }

    /**
     * insère un log entier en début de liste
     */
    public insertLog(l: cLog) {
        for (let i = l.messages.length - 1; i >= 0; i--) {
            this.insert(l.messages[i]);
        }
    }

    public get messages() {
        return this._messages;
    }

    public toString(): string {
        return this._messages.join("\n");
    }

    /**
     * @param mc message code you're looking for
     * @returns true if log contains at least one occurrence of given message code
     */
    public contains(mc: MessageCode): boolean {
        for (const m of this.messages) {
            if (m.code === mc) {
                return true;
            }
        }
        return false;
    }

    /**
     * compute error, warning, info count in a message list
     */
    public static messagesStats(messages: Message[], stats?: any): any {
        if (stats === undefined) {
            stats = { info: 0, warning: 0, error: 0 };
        }
        for (const m of messages) {
            const s = m.getSeverity();
            switch (s) {
                case MessageSeverity.INFO:
                    stats.info++;
                    break;
                case MessageSeverity.WARNING:
                    stats.warning++;
                    break;
                case MessageSeverity.ERROR:
                    stats.error++;
                    break;
            }
        }
        return stats;
    }
}
